@@ -2,7 +2,7 @@ from __future__ import unicode_literals
 
 __author__ = "Shyue Ping Ong, William Davidson Richards, Stephen Dacek, " \
              "Xiaohui Qu"
-__date__ = "Sep 22 2014"
-__version__ = "0.8.1"
+__date__ = "Aug 2 2015"
+__version__ = "0.8.4"
 
 from .custodian import Custodian
