@@ -1,10 +1,9 @@
-#!/usr/bin/env python
+from __future__ import unicode_literals, division
 
 """
 TODO: Modify module doc.
 """
 
-from __future__ import division
 
 __author__ = "Shyue Ping Ong"
 __copyright__ = "Copyright 2012, The Materials Project"
@@ -12,4 +11,3 @@ __version__ = "0.1"
 __maintainer__ = "Shyue Ping Ong"
 __email__ = "shyuep@gmail.com"
 __date__ = "5/3/13"
-
