@@ -35,15 +35,16 @@ from math import ceil
 
 from custodian.custodian import ErrorHandler
 from custodian.utils import backup
-from pymatgen.io.vaspio.vasp_input import Poscar, VaspInput, Incar, Kpoints
+from pymatgen.io.vasp import Poscar, VaspInput, Incar, Kpoints, Vasprun, Oszicar
 from pymatgen.transformations.standard_transformations import \
     SupercellTransformation
 
-from pymatgen.io.vaspio.vasp_output import Vasprun, Oszicar
 from custodian.ansible.interpreter import Modder
 from custodian.ansible.actions import FileActions
 from custodian.vasp.interpreter import VaspModder
 
+VASP_BACKUP_FILES = {"INCAR", "KPOINTS", "POSCAR", "OUTCAR", "OSZICAR",
+                     "vasprun.xml", "vasp.out"}
 
 class VaspErrorHandler(ErrorHandler):
     """
@@ -76,12 +77,11 @@ class VaspErrorHandler(ErrorHandler):
         "pricel": ["internal error in subroutine PRICEL"],
         "zpotrf": ["LAPACK: Routine ZPOTRF failed"],
         "amin": ["One of the lattice vectors is very long (>50 A), but AMIN"],
-        "zbrent": ["ZBRENT: fatal internal in"],
-        "aliasing": ["WARNING: small aliasing (wrap around) errors must be expected"],
-        "aliasing_incar": ["Your FFT grids (NGX,NGY,NGZ) are not sufficient "
-                           "for an accurate"],
+        "zbrent": ["ZBRENT: fatal internal in",
+                   "ZBRENT: fatal error in bracketing"],
         "pssyevx": ["ERROR in subspace rotation PSSYEVX"],
-        "eddrmm": ["WARNING in EDDRMM: call to ZHEGV failed"]
+        "eddrmm": ["WARNING in EDDRMM: call to ZHEGV failed"],
+        "edddav": ["Error EDDDAV: Call to ZHEGV failed"]
     }
 
     def __init__(self, output_filename="vasp.out"):
@@ -116,8 +116,7 @@ class VaspErrorHandler(ErrorHandler):
         return len(self.errors) > 0
 
     def correct(self):
-        backup([self.output_filename, "INCAR", "KPOINTS", "POSCAR", "OUTCAR",
-                "OSZICAR", "vasprun.xml"])
+        backup(VASP_BACKUP_FILES | {self.output_filename})
         actions = []
         vi = VaspInput.from_directory(".")
 
@@ -132,6 +131,11 @@ class VaspErrorHandler(ErrorHandler):
         if "brmix" in self.errors:
             actions.append({"dict": "INCAR",
                             "action": {"_set": {"ISYM": 0}}})
+
+            if vi["KPOINTS"].style == Kpoints.supported_modes.Monkhorst:
+                actions.append({"dict": "KPOINTS",
+                                "action": {"_set": {"generation_style": "Gamma"}}})
+
             # Based on VASP forum's recommendation, you should delete the
             # CHGCAR and WAVECAR when dealing with this error.
             actions.append({"file": "CHGCAR",
@@ -174,8 +178,10 @@ class VaspErrorHandler(ErrorHandler):
                             "action": {"_set": {"LREAL": False}}})
 
         if self.errors.intersection(["tetirr", "incorrect_shift"]):
-            actions.append({"dict": "KPOINTS",
-                            "action": {"_set": {"generation_style": "Gamma"}}})
+
+            if vi["KPOINTS"].style == Kpoints.supported_modes.Monkhorst:
+                actions.append({"dict": "KPOINTS",
+                                "action": {"_set": {"generation_style": "Gamma"}}})
 
         if "rot_matrix" in self.errors:
             if vi["KPOINTS"].style == Kpoints.supported_modes.Monkhorst:
@@ -226,6 +232,83 @@ class VaspErrorHandler(ErrorHandler):
             actions.append({"dict": "INCAR",
                             "action": {"_set": {"NBANDS": int(1.1 * nbands)}}})
 
+        if "pssyevx" in self.errors:
+            actions.append({"dict": "INCAR", "action":
+                                    {"_set": {"ALGO": "Normal"}}})
+        if "eddrmm" in self.errors:
+            #RMM algorithm is not stable for this calculation
+            if vi["INCAR"].get("ALGO", "Normal") in ["Fast", "VeryFast"]:
+                actions.append({"dict": "INCAR", "action":
+                                        {"_set": {"ALGO": "Normal"}}})
+            else:
+                potim = float(vi["INCAR"].get("POTIM", 0.5)) / 2.0
+                actions.append({"dict": "INCAR",
+                                "action": {"_set": {"POTIM": potim}}})
+
+            actions.append({"file": "CHGCAR",
+                            "action": {"_file_delete": {'mode': "actual"}}})
+            actions.append({"file": "WAVECAR",
+                            "action": {"_file_delete": {'mode': "actual"}}})
+        if "edddav" in self.errors:
+            actions.append({"file": "CHGCAR",
+                            "action": {"_file_delete": {'mode': "actual"}}})
+            actions.append({"dict": "INCAR", "action":
+                            {"_set": {"ALGO": "All"}}})
+
+        VaspModder(vi=vi).apply_actions(actions)
+        return {"errors": list(self.errors), "actions": actions}
+
+
+class AliasingErrorHandler(ErrorHandler):
+    """
+    Master VaspErrorHandler class that handles a number of common errors
+    that occur during VASP runs.
+    """
+
+    is_monitor = True
+
+    error_msgs = {
+        "aliasing": ["WARNING: small aliasing (wrap around) errors must be expected"],
+        "aliasing_incar": ["Your FFT grids (NGX,NGY,NGZ) are not sufficient "
+                           "for an accurate"]
+    }
+
+    def __init__(self, output_filename="vasp.out"):
+        """
+        Initializes the handler with the output file to check.
+
+        Args:
+            output_filename (str): This is the file where the stdout for vasp
+                is being redirected. The error messages that are checked are
+                present in the stdout. Defaults to "vasp.out", which is the
+                default redirect used by :class:`custodian.vasp.jobs.VaspJob`.
+        """
+        self.output_filename = output_filename
+        self.errors = set()
+
+    def check(self):
+        incar = Incar.from_file("INCAR")
+        self.errors = set()
+        with open(self.output_filename, "r") as f:
+            for line in f:
+                l = line.strip()
+                for err, msgs in AliasingErrorHandler.error_msgs.items():
+                    for msg in msgs:
+                        if l.find(msg) != -1:
+                            # this checks if we want to run a charged
+                            # computation (e.g., defects) if yes we don't
+                            # want to kill it because there is a change in e-
+                            # density (brmix error)
+                            if err == "brmix" and 'NELECT' in incar:
+                                continue
+                            self.errors.add(err)
+        return len(self.errors) > 0
+
+    def correct(self):
+        backup(VASP_BACKUP_FILES | {self.output_filename})
+        actions = []
+        vi = VaspInput.from_directory(".")
+
         if "aliasing" in self.errors:
             with open("OUTCAR") as f:
                 grid_adjusted = False
@@ -257,24 +340,6 @@ class VaspErrorHandler(ErrorHandler):
                             {"file": "WAVECAR",
                              "action": {"_file_delete": {'mode': "actual"}}}])
 
-        if "pssyevx" in self.errors:
-            actions.append({"dict": "INCAR", "action":
-                                    {"_set": {"ALGO": "Normal"}}})
-        if "eddrmm" in self.errors:
-            #RMM algorithm is not stable for this calculation
-            if vi["INCAR"].get("ALGO", "Normal") in ["Fast", "VeryFast"]:
-                actions.append({"dict": "INCAR", "action":
-                                        {"_set": {"ALGO": "Normal"}}})
-            else:
-                potim = float(vi["INCAR"].get("POTIM", 0.5)) / 2.0
-                actions.append({"dict": "INCAR",
-                                "action": {"_set": {"POTIM": potim}}})
-
-            actions.append({"file": "CHGCAR",
-                            "action": {"_file_delete": {'mode': "actual"}}})
-            actions.append({"file": "WAVECAR",
-                            "action": {"_file_delete": {'mode': "actual"}}})
-
         VaspModder(vi=vi).apply_actions(actions)
         return {"errors": list(self.errors), "actions": actions}
 
@@ -306,11 +371,18 @@ class MeshSymmetryErrorHandler(ErrorHandler):
     def check(self):
         msg = "Reciprocal lattice and k-lattice belong to different class of" \
               " lattices."
+
+        vi = VaspInput.from_directory('.')
+        # According to VASP admins, you can disregard this error
+        # if symmetry is off
+        #Also disregard if automatic KPOINT generation is used
+        if (not vi["INCAR"].get('ISYM', True)) or \
+                    vi["KPOINTS"].style == Kpoints.supported_modes.Automatic:
+            return False
+
         try:
             v = Vasprun(self.output_vasprun)
-            # According to VASP admins, you can disregard this error
-            # if symmetry is off
-            if v.converged or (not v.incar.get('ISYM', True)):
+            if v.converged:
                 return False
         except:
             pass
@@ -322,8 +394,7 @@ class MeshSymmetryErrorHandler(ErrorHandler):
         return False
 
     def correct(self):
-        backup([self.output_filename, "INCAR", "KPOINTS", "POSCAR", "OUTCAR",
-                "vasprun.xml"])
+        backup(VASP_BACKUP_FILES | {self.output_filename})
         vi = VaspInput.from_directory(".")
         m = reduce(operator.mul, vi["KPOINTS"].kpts[0])
         m = max(int(round(m ** (1 / 3))), 1)
@@ -361,8 +432,7 @@ class UnconvergedErrorHandler(ErrorHandler):
         return False
 
     def correct(self):
-        backup(["INCAR", "KPOINTS", "POSCAR", "OUTCAR", "vasprun.xml", 
-                "vasp.out"])
+        backup(VASP_BACKUP_FILES)
         actions = [{"file": "CONTCAR",
                     "action": {"_file_copy": {"dest": "POSCAR"}}},
                    {"dict": "INCAR",
@@ -409,8 +479,7 @@ class MaxForceErrorHandler(ErrorHandler):
         return False
 
     def correct(self):
-        backup(["INCAR", "KPOINTS", "POSCAR", "OUTCAR",
-                self.output_filename, "vasp.out"])
+        backup(VASP_BACKUP_FILES | {self.output_filename})
         vi = VaspInput.from_directory(".")
         ediff = float(vi["INCAR"].get("EDIFF", 1e-4))
         actions = [{"file": "CONTCAR",
@@ -459,8 +528,7 @@ class PotimErrorHandler(ErrorHandler):
             return False
 
     def correct(self):
-        backup(["INCAR", "KPOINTS", "POSCAR", "OUTCAR",
-                "vasprun.xml"])
+        backup(VASP_BACKUP_FILES)
         vi = VaspInput.from_directory(".")
         potim = float(vi["INCAR"].get("POTIM", 0.5))
         ibrion = int(vi["INCAR"].get("IBRION", 0))
@@ -506,8 +574,7 @@ class FrozenJobErrorHandler(ErrorHandler):
             return True
 
     def correct(self):
-        backup([self.output_filename, "INCAR", "KPOINTS", "POSCAR", "OUTCAR",
-                "vasprun.xml"])
+        backup(VASP_BACKUP_FILES | {self.output_filename})
 
         vi = VaspInput.from_directory('.')
         actions = []
@@ -572,20 +639,20 @@ class NonConvergingErrorHandler(ErrorHandler):
         actions = []
         if self.change_algo:
             if algo == "Fast":
-                backup(["INCAR", "KPOINTS", "POSCAR", "OUTCAR", "vasprun.xml"])
+                backup(VASP_BACKUP_FILES)
                 actions.append({"dict": "INCAR",
                             "action": {"_set": {"ALGO": "Normal"}}})
 
             elif amix > 0.1 and bmix > 0.01:
                 #try linear mixing
-                backup(["INCAR", "KPOINTS", "POSCAR", "OUTCAR", "vasprun.xml"])
+                backup(VASP_BACKUP_FILES)
                 actions.append({"dict": "INCAR",
                                 "action": {"_set": {"AMIX": 0.1, "BMIX": 0.01,
                                                     "ICHARG": 2}}})
 
             elif bmix < 3.0 and amin > 0.01:
                 #Try increasing bmix
-                backup(["INCAR", "KPOINTS", "POSCAR", "OUTCAR", "vasprun.xml"])
+                backup(VASP_BACKUP_FILES)
                 actions.append({"dict": "INCAR",
                                 "action": {"_set": {"AMIN": 0.01, "BMIX": 3.0,
                                                     "ICHARG": 2}}})
@@ -858,7 +925,7 @@ class PositiveEnergyErrorHandler(ErrorHandler):
         vi = VaspInput.from_directory(".")
         algo = vi["INCAR"].get("ALGO", "Normal")
         if algo.lower() not in ['normal', 'n']:
-            backup(["INCAR", "KPOINTS", "POSCAR", "OUTCAR", "vasprun.xml"])
+            backup(VASP_BACKUP_FILES)
             actions = [{"dict": "INCAR",
                         "action": {"_set": {"ALGO": "Normal"}}}]
             VaspModder(vi=vi).apply_actions(actions)
