@@ -21,14 +21,12 @@ import shutil
 import math
 import logging
 
-from pymatgen.io.vaspio.vasp_input import VaspInput, Incar
-from pymatgen.io.vaspio.vasp_output import Outcar
-from pymatgen.io.smartio import read_structure
-from pymatgen.io.vaspio_set import MITVaspInputSet
+from pymatgen.io.vasp import VaspInput, Incar, Poscar, Outcar
+from pymatgen.io.smart import read_structure
+from pymatgen.io.vasp.sets import MITVaspInputSet
 from monty.json import MontyDecoder
 from monty.os.path import which
 
-from custodian.ansible.actions import FileActions, DictActions
 from custodian.custodian import Job, gzip_dir
 from custodian.vasp.interpreter import VaspModder
 
@@ -73,7 +71,7 @@ class VaspJob(Job):
             gzipped (bool): Deprecated. Please use the Custodian class's
                 gzipped_output option instead.
             default_vasp_input_set (VaspInputSet): Species the default input
-                set (see pymatgen's documentation in pymatgen.io.vaspio_set to
+                set (see pymatgen's documentation in pymatgen.io.vasp.sets to
                 use for directories that do not contain full set of VASP
                 input files. For example, if a directory contains only a
                 POSCAR or a cif, the vasp input set will be used to generate
@@ -156,7 +154,10 @@ class VaspJob(Job):
                         del incar["NPAR"]
                     else:
                         import multiprocessing
-                        ncores = multiprocessing.cpu_count()
+                        # try sge environment variable first
+                        # (since multiprocessing counts cores on the current machine only)
+                        ncores = os.environ.get('NSLOTS') or multiprocessing.cpu_count()
+                        ncores = int(ncores)
                         for npar in range(int(round(math.sqrt(ncores))),
                                           ncores):
                             if ncores % npar == 0:
@@ -216,8 +217,8 @@ class VaspJob(Job):
         if self.gzipped:
             gzip_dir(".")
 
-    @staticmethod
-    def double_relaxation_run(vasp_cmd, gzipped=True):
+    @classmethod
+    def double_relaxation_run(cls, vasp_cmd, auto_npar=True):
         """
         Returns a list of two jobs corresponding to an AFLOW style double
         relaxation run.
@@ -226,20 +227,74 @@ class VaspJob(Job):
             vasp_cmd (str): Command to run vasp as a list of args. For example,
                 if you are using mpirun, it can be something like
                 ["mpirun", "pvasp.5.2.11"]
+            auto_npar (bool): Whether to automatically tune NPAR to be sqrt(
+                number of cores) as recommended by VASP for DFT calculations.
+                Generally, this results in significant speedups. Defaults to
+                True. Set to False for HF, GW and RPA calculations.
 
         Returns:
             List of two jobs corresponding to an AFLOW style run.
         """
-        return [VaspJob(vasp_cmd, final=False, suffix=".relax1"),
+        return [VaspJob(vasp_cmd, final=False, suffix=".relax1",
+                        auto_npar=auto_npar),
                 VaspJob(
                     vasp_cmd, final=True, backup=False,
-                    suffix=".relax2", gzipped=gzipped,
+                    suffix=".relax2", gzipped=gzipped, auto_npar=auto_npar,
                     settings_override=[
                         {"dict": "INCAR",
                          "action": {"_set": {"ISTART": 1}}},
                         {"file": "CONTCAR",
                          "action": {"_file_copy": {"dest": "POSCAR"}}}])]
 
+    @classmethod
+    def full_opt_run(cls, vasp_cmd, auto_npar=True, vol_change_tol=0.05,
+                     max_steps=10):
+        """
+        Returns a generator of jobs for a full optimization run. Basically,
+        this runs an infinite series of geometry optimization jobs until the
+        % vol change in a particular optimization is less than vol_change_tol.
+
+        Args:
+            vasp_cmd (str): Command to run vasp as a list of args. For example,
+                if you are using mpirun, it can be something like
+                ["mpirun", "pvasp.5.2.11"]
+            auto_npar (bool): Whether to automatically tune NPAR to be sqrt(
+                number of cores) as recommended by VASP for DFT calculations.
+                Generally, this results in significant speedups. Defaults to
+                True. Set to False for HF, GW and RPA calculations.
+            vol_change_tol (float): The tolerance at which to stop a run.
+                Defaults to 0.05, i.e., 5%.
+            max_steps (int): The maximum number of runs. Defaults to 10 (
+                highly unlikely that this limit is ever reached).
+
+        Returns:
+            Generator of jobs.
+        """
+        for i in xrange(max_steps):
+            if i == 0:
+                settings = None
+                backup = True
+            else:
+                backup = False
+                initial = Poscar.from_file("POSCAR").structure
+                final = Poscar.from_file("CONTCAR").structure
+                vol_change = (final.volume - initial.volume) / initial.volume
+
+                logging.info("Vol change = %.1f %%!" % (vol_change * 100))
+                if abs(vol_change) < vol_change_tol:
+                    logging.info("Stopping optimization!")
+                    break
+                else:
+                    settings = [
+                        {"dict": "INCAR",
+                         "action": {"_set": {"ISTART": 1}}},
+                        {"file": "CONTCAR",
+                         "action": {"_file_copy": {"dest": "POSCAR"}}}]
+            logging.info("Generating job = %d!" % (i+1))
+            yield VaspJob(vasp_cmd, final=False, backup=backup,
+                          suffix=".relax%d" % (i+1),
+                          settings_override=settings)
+
     def as_dict(self):
         d = dict(vasp_cmd=self.vasp_cmd,
                  output_file=self.output_file, suffix=self.suffix,
